package jdk.internal.org.objectweb.asm.util;

import jdk.internal.org.objectweb.asm.MethodVisitor;
import jdk.internal.org.objectweb.asm.FieldVisitor;
import jdk.internal.org.objectweb.asm.Attribute;
import jdk.internal.org.objectweb.asm.TypePath;
import jdk.internal.org.objectweb.asm.AnnotationVisitor;
import java.io.PrintWriter;
import jdk.internal.org.objectweb.asm.ClassVisitor;

public final class TraceClassVisitor extends ClassVisitor
{
    private final PrintWriter pw;
    public final Printer p;
    
    public TraceClassVisitor(PrintWriter printWriter) {
        this(null, printWriter);
    }
    
    public TraceClassVisitor(ClassVisitor classVisitor, PrintWriter printWriter) {
        this(classVisitor, new Textifier(), printWriter);
    }
    
    public TraceClassVisitor(ClassVisitor classVisitor, Printer p3, PrintWriter pw) {
        super(327680, classVisitor);
        this.pw = pw;
        this.p = p3;
    }
    
    @Override
    public void visit(int version, int access, String name, String signature, String superName, String[] interfaces) {
        this.p.visit(version, access, name, signature, superName, interfaces);
        super.visit(version, access, name, signature, superName, interfaces);
    }
    
    @Override
    public void visitSource(String source, String debug) {
        this.p.visitSource(source, debug);
        super.visitSource(source, debug);
    }
    
    @Override
    public void visitOuterClass(String owner, String name, String descriptor) {
        this.p.visitOuterClass(owner, name, descriptor);
        super.visitOuterClass(owner, name, descriptor);
    }
    
    @Override
    public AnnotationVisitor visitAnnotation(String descriptor, boolean visible) {
        return new TraceAnnotationVisitor((this.cv == null) ? null : this.cv.visitAnnotation(descriptor, visible), this.p.visitClassAnnotation(descriptor, visible));
    }
    
    @Override
    public AnnotationVisitor visitTypeAnnotation(int typeRef, TypePath typePath, String descriptor, boolean visible) {
        return new TraceAnnotationVisitor((this.cv == null) ? null : this.cv.visitTypeAnnotation(typeRef, typePath, descriptor, visible), this.p.visitClassTypeAnnotation(typeRef, typePath, descriptor, visible));
    }
    
    @Override
    public void visitAttribute(Attribute attribute) {
        this.p.visitClassAttribute(attribute);
        super.visitAttribute(attribute);
    }
    
    @Override
    public void visitInnerClass(String name, String outerName, String innerName, int access) {
        this.p.visitInnerClass(name, outerName, innerName, access);
        super.visitInnerClass(name, outerName, innerName, access);
    }
    
    @Override
    public FieldVisitor visitField(int access, String name, String descriptor, String signature, Object value) {
        return new TraceFieldVisitor((this.cv == null) ? null : this.cv.visitField(access, name, descriptor, signature, value), this.p.visitField(access, name, descriptor, signature, value));
    }
    
    @Override
    public MethodVisitor visitMethod(int access, String name, String descriptor, String signature, String[] exceptions) {
        return new TraceMethodVisitor((this.cv == null) ? null : this.cv.visitMethod(access, name, descriptor, signature, exceptions), this.p.visitMethod(access, name, descriptor, signature, exceptions));
    }
    
    @Override
    public void visitEnd() {
        this.p.visitClassEnd();
        if (this.pw != null) {
            this.p.print(this.pw);
            this.pw.flush();
        }
        super.visitEnd();
    }
}
